
library(mgcv)
library(jpeg)
library(spatstat)
library(ggplot2)
library(grid)

folder <- 'fixloc_1/fixloc_'

imrange <- read.table('imrange.dat',header=TRUE)
xrange <- imrange$xrange
yrange <- imrange$yrange


bws <- seq(.7,1.6,length=37)
ltotal <- matrix(data=0,nrow=length(bws),ncol=1)

kbw <- 0
pb <- txtProgressBar(min = 0, max = length(bws), initial = 0, char = "=", style = 3)
for (bw in bws){
  kbw<- kbw+1
n <- 0
lImage <-matrix(data = 0, nrow = 30, ncol = 1)
nImage <-matrix(data = 0, nrow = 30, ncol = 1)
for (iImage in (c(1:5,16:20))) {
  data <- read.table(paste(folder,toString(iImage),".dat",sep=""),header=TRUE)
  lsub <- matrix(data = 0, nrow = length(unique(data$subj)), ncol = 1)
  data <- subset(data,subj<=15)
  for (iSub in unique(data$subj)){
    datWithoutSub <- subset(data,subj!=iSub)
    datSub <- subset(data,subj==iSub)
    Xppp <- ppp(datWithoutSub$xpos,datWithoutSub$ypos,xrange=xrange,yrange=yrange)
    #sal  <- density(Xppp,(2/3+2/3*(bw-1)/4))
    #sal  <- density(Xppp,mean(bw.scott(Xppp)))
    sal <- density(Xppp,bw)
    sal$v<- sal$v/sum(sal$v)
    lfix <- matrix(data = 1, nrow = length(datSub$xpos), ncol = 1)
    for (ifix in (1:length(datSub$xpos))) {
      if (datSub$xpos[ifix] < sal$xrange[2] && datSub$xpos[ifix]>sal$xrange[1] && datSub$ypos[ifix] < sal$yrange[2] && datSub$ypos[ifix]>sal$yrange[1]){
        idx <- round((datSub$xpos[ifix]-sal$xrange[1])/sal$xstep+0.5);
        idy <- round((datSub$ypos[ifix]-sal$yrange[1])/sal$ystep+0.5);
        lfix[ifix] <- sal$v[idy,idx]
      }
    }
    lsub[iSub] = sum(log2(lfix));
    n <- n +length(lfix);
    nImage[iImage] <- nImage[iImage] +length(lfix)
  }
  lImage[iImage] <- sum(lsub)
}
ltotal[kbw] <- sum(lImage)/n
setTxtProgressBar(pb, kbw)
}
lInhom <- max(ltotal,na.rm = TRUE)
bestBw <-bws[which.max(ltotal)]

pdata <- data.frame(bws = bws,l = ltotal)
pmax  <- data.frame(bws = c(bestBw,bestBw),l = c(-13.425,lInhom))

ggplot(pdata,aes(x=bws,y=l))+geom_path()+geom_point()+
  xlab("Bandwidth [°]") + ylab("Log-Likelihood / Fixation [bit / fix]") + 
  theme(text = element_text(size=20),panel.grid.minor=element_blank()) +
  geom_path(data=pmax,arrow = arrow())


## calculate the training-> test prediction of saliency

n <- 0
lImage <-matrix(data = 0, nrow = 30, ncol = 1)
nImage <-matrix(data = 0, nrow = 30, ncol = 1)
for (iImage in (c(6:15))) {
  data <- read.table(paste(folder,toString(iImage),".dat",sep=""),header=TRUE)
  lsub <- matrix(data = 0, nrow = length(unique(data$subj)), ncol = 1)
  dat  <- subset(data,subj<=15)
  Xppp <- ppp(dat$xpos,dat$ypos,xrange=xrange,yrange=yrange)
  sal <- density(Xppp,bestBw)
  sal$v<- sal$v/sum(sal$v)
  for (iSub in (16:35)){
    datSub <- subset(data,subj==iSub)
    lfix <- matrix(data = 1, nrow = length(datSub$xpos), ncol = 1)
    for (ifix in (1:length(datSub$xpos))) {
      if (datSub$xpos[ifix] < sal$xrange[2] && datSub$xpos[ifix]>sal$xrange[1] && datSub$ypos[ifix] < sal$yrange[2] && datSub$ypos[ifix]>sal$yrange[1]){
        idx <- round((datSub$xpos[ifix]-sal$xrange[1])/sal$xstep+0.5);
        idy <- round((datSub$ypos[ifix]-sal$yrange[1])/sal$ystep+0.5);
        lfix[ifix] <- sal$v[idy,idx]
      }
    }
    lsub[iSub] = sum(log2(lfix));
    n <- n +length(lfix);
    nImage[iImage] <- nImage[iImage] +length(lfix)
  }
  lImage[iImage] <- sum(lsub)
}
lInhomTest <- sum(lImage)/n


lhom <- -14

dAll<- read.table(paste(folder,toString(1),".dat",sep=""),header=TRUE)
dAll$Image <- 1
for (iImage in (2:30)) {
  data <- read.table(paste(folder,toString(iImage),".dat",sep=""),header=TRUE)
  data$Image <- iImage
  dAll<- rbind(dAll,data)
}
lfix <- matrix(data = 1, nrow = length(dAll$xpos), ncol = 1)
for (iImage in (1:30)) {
  dAllW  <- subset(dAll, Image != iImage)
  dImage <- subset(dAll, Image == iImage)
  Allppp <- ppp(dAllW$xpos,dAllW$ypos,xrange=xrange,yrange=yrange)
  Cbias <- density(Allppp,bestBw)
  Cbias$v<-Cbias$v/sum(Cbias$v)
  lImage <- matrix(data = 1, nrow = length(dImage$xpos), ncol = 1)
  for (ifix in (1:length(dImage$xpos))) {
    if (dImage$xpos[ifix] < Cbias$xrange[2] && dImage$xpos[ifix]>Cbias$xrange[1] && dImage$ypos[ifix] < Cbias$yrange[2] && dImage$ypos[ifix]>Cbias$yrange[1]){
      idx <- round((dImage$xpos[ifix]-Cbias$xrange[1])/Cbias$xstep+0.5);
      idy <- round((dImage$ypos[ifix]-Cbias$yrange[1])/Cbias$ystep+0.5);
      lImage[ifix] <- Cbias$v[idy,idx]
    }
  }
  lfix[dAll$Image==iImage]<- lImage
}
minL <- mean(log2(lfix))