function CVstruct = crossValidate(modelNumber)

switch modelNumber
    case 1 % Full subtractive original
        nvars = 8;
        modelType = 2;
    case 2 % Gamma fixed subtractive original
        nvars = 7;
        modelType = 4;
    case 3 % Lambda fixed subtractive original
        nvars = 7;
        modelType = 5;
    case 4 % All fixed subtractive original
        nvars = 6;
        modelType = 3;
    case 5 % Full divisive
        nvars = 8;
        modelType = 2;
    case 6 % Gamma fixed divisive
        nvars = 7;
        modelType = 4;
    case 7 % Lambda fixed divisive
        nvars = 7;
        modelType = 5;
    case 8 % All fixed divisive
        nvars = 6;
        modelType = 3;
    case 9 % No inhibition with lambda
        nvars = 4;
        modelType = 2;
    case 10% No inhibition and no exponent
        nvars = 3;
        modelType = 3;
end

if exist(sprintf('crossvalidation/CVstruct%02d.mat',modelNumber),'file')
    load(sprintf('crossvalidation/CVstruct%02d.mat',modelNumber));
else
    CVstruct = [];
end


for iFold = 1:5
    if numel(CVstruct)<iFold
        fprintf('starting fold %d for model # %d\n',iFold,modelNumber)
        testSubs = (iFold-1)*7+(1:7);
        trainSubs = setdiff(1:35,testSubs);
        testImages = [(iFold-1)*3+(1:3),15+(iFold-1)*3+(1:3)];
        trainImages = setdiff(1:30,testImages);
        dataset = 10+iFold;
        switch modelNumber
            case {1,2,3,4}
                [x,fval,exitflag,output,population,score] = geneticAlgorithm(nvars,modelType,trainImages,trainSubs,dataset);
            case {5,6,7,8}
                [x,fval,exitflag,output,population,score] = geneticAlgorithm_divisive(nvars,modelType,trainImages,trainSubs,dataset);
            case {9,10}
                [x,fval,exitflag,output,population,score] = geneticAlgorithm_noInhib(nvars,modelType,trainImages,trainSubs,dataset);
        end
        CVstruct(iFold).nvars = nvars;
        CVstruct(iFold).modelNumber = modelNumber;
        CVstruct(iFold).modelType = modelType;
        CVstruct(iFold).dataset = dataset;
        CVstruct(iFold).pars = x;
        CVstruct(iFold).exitflag = exitflag;
        CVstruct(iFold).output = output;
        CVstruct(iFold).population = population;
        CVstruct(iFold).score = score;
        pT = exp(x);
        CVstruct(iFold).parsTransformed = pT;
        CVstruct(iFold).trainPosterior = fval;
        switch modelNumber
            case {1,2,3,4}
                [CVstruct(iFold).likelihoodTrain,CVstruct(iFold).NTrain] = dynamic_likelihood_derivative(x,modelType,trainImages,trainSubs,dataset);
                [CVstruct(iFold).likelihoodTest,CVstruct(iFold).NTest] = dynamic_likelihood_derivative(x,modelType,testImages,testSubs,dataset);
            case {5,6,7,8}
                [CVstruct(iFold).likelihoodTrain,CVstruct(iFold).NTrain] = dynamic_likelihood_divisive(x,modelType,trainImages,trainSubs,dataset);
                [CVstruct(iFold).likelihoodTest,CVstruct(iFold).NTest] = dynamic_likelihood_divisive(x,modelType,testImages,testSubs,dataset);
            case {9,10}
                [CVstruct(iFold).likelihoodTrain,CVstruct(iFold).NTrain] = dynamic_likelihood_noInhib(x,modelType,trainImages,trainSubs,dataset);
                [CVstruct(iFold).likelihoodTest,CVstruct(iFold).NTest] = dynamic_likelihood_noInhib(x,modelType,testImages,testSubs,dataset);
        end
        CVstruct(iFold).bitperfixation = CVstruct(iFold).likelihoodTest./log(2)./CVstruct(iFold).NTest;
        save(sprintf('crossvalidation/CVstruct%02d.mat',modelNumber),'CVstruct')
    end
end
