function [fixations,likelihood,LGrad,mapsA,mapsI,us] = dynamic_model_derivative(im_size,deg_size,saliency,fix0,times,fixations,pars,trialTime,nfixations,options)
% function [fixations,likelihood,LGrad,mapsA,mapsI,us] = dynamic_model_derivative(im_size,deg_size,saliency,fix0,times,fixations,pars,trialTime,nfixations,options)
% This is the version with derivatives.
% I removed the complexity of the timeshift for now.
% When more than 2 outputs are asked for, the gradient is returned for the
% 9 parameters. 

% ifix and ichoice could be unified in this function, but I left them for
% now.

%rng('shuffle') % takes some time 
if ~exist('options','var')
    options = struct;
end

if ~isfield(options,'targetSelector') || isempty(options.targetSelector)
    options.targetSelector = 3;
end

if ~isfield(options,'normalizeInhib')
    options.normalizeInhib = true;
end

if ~isfield(options,'nonzero')
    options.nonzero = true;
end

gammaPar1 = 9;
gammaPar2 = 0.275/9;

%% read input/set parameters
if ~exist('times','var') || isempty(times)
    if ~exist('trialTime','var') || isempty(trialTime)
        if ~exist('nfixations','var') || isempty(nfixations)
            nfixations = 100;
        end
        times      = gamrnd(gammaPar1,gammaPar2,nfixations,1); % not terribly accurate...  gamma distribution
    else
        nfixations = inf;
        %times      = [];
        times      = gamrnd(gammaPar1,gammaPar2,1,1);
    end
else
    nfixations = length(times);
    times(isnan(times)) = gamrnd(gammaPar1,gammaPar2,sum(isnan(times(:))),1);
end

if ~exist('trialTime','var') || isempty(trialTime)
    trialTime = inf;
end
if exist('fixations','var') && ~isempty(fixations)
    assert(size(fixations,1) == (length(times)-1), 'number of times and fixations need to be consistent')
    fixations = [fix0;fixations];
else
    if isfinite(nfixations)
        fixations = nan(nfixations+1,2);
    else
        fixations = [];
    end
end

assert(all(size(saliency)==im_size),'image size and size of the saliency map should be equal');

if exist('pars','var') && ~isempty(pars)
    omega          = pars(1);
    rho            = pars(2);
    sigmaAttention = pars(3);
    sigmaInhib     = pars(4);
    gamma          = pars(5);
    lambda         = pars(6);
    inhibStrength  = pars(7);
    zeta           = pars(8);
else
    omega          = 10^(-4.1722)*100;
    rho            = 10^(-0.8343)*100;
    sigmaAttention = 6.1036;
    sigmaInhib     = 2.1319;
    gamma          = .3;
    lambda         = 1;
    inhibStrength  = 1;
    zeta           = 0.04;
end

if options.nonzero
    initInhib      = 1./prod(im_size);
else
    initInhib = 0;
end
%% initialize

positionx = linspace(0,1,im_size(2)).*deg_size(2);
positiony = (linspace(0,1,im_size(1)).*deg_size(1))';





%% start calculations
fixation = fix0;
fixations(1,:)=fixation;
mapAtt   = eps.*ones(im_size); % set to minimal value, but > 0 to ensure working for 0 time
%mapInhib = ones(im_size)./prod(im_size); % normalized to sum = 1
mapInhib = ones(im_size)*initInhib;
maxInhib       = max(mapInhib(:));
mapInhib       = mapInhib./ maxInhib ; %normalize for numerical reasons
fixidx   = [];

if nargout > 2
    LGrad = zeros(8,1);
    AdSigma = zeros(size(mapAtt));
    AdOmega = zeros(size(mapAtt));
    FdSigma = zeros(size(mapAtt));
    FdOmega = zeros(size(mapAtt));
end

if nargout> 3 && isfinite(nfixations)
    mapsA = nan([im_size,nfixations]);
    mapsI = nan([im_size,nfixations]);
    us    = nan([im_size,nfixations]);
end
if isfinite(nfixations)
    likelihood = NaN(nfixations-1,1);
else
    likelihood = [];
end
ifix = 1;
ichoice = 1;

while ichoice < nfixations && sum(times) <= trialTime
    time           = times(ifix);
    rad            = -(bsxfun(@plus,(positionx-fixation(1)).^2,(positiony-fixation(2)).^2));% to calculate gaussians (used 2 times)
    gaussAttention = exp(rad./(sigmaAttention.^2)./2);
    gaussInhib     = exp(rad./(sigmaInhib.^2)./2);
    salFixation    = saliency.*gaussAttention;
    salNormalizer  = sum(salFixation(:));
    salFixation    = salFixation./sum(salFixation(:)); % normalization pre-dynamics
    gaussInhib     = gaussInhib./sum(gaussInhib(:));
    if nargout >2
        AdOmega = -time.*exp(-time.*rho).*(mapAtt-salFixation)+exp(-time.*rho).*AdOmega;
        FdOmega = -time.*exp(-time.*omega).*(maxInhib.*mapInhib-gaussInhib)+exp(-time.*omega).*FdOmega;
    end
    mapAtt         = salFixation   + exp(-time.*rho  ).*(mapAtt   - salFixation); % solutions to linear differential equations
    mapInhib       = gaussInhib    + exp(-time.*omega).*(maxInhib.*mapInhib - gaussInhib);
    maxInhib       = max(mapInhib(:));
    mapInhib       = mapInhib./ maxInhib ; %normalize for numerical reasons
    ifix = ifix + 1;
    if ~isfinite(nfixations)
        times(ifix)    = gamrnd(gammaPar1,gammaPar2,1,1); % not terribly accurate... gamma distribution
    end
    mapAttPower    = exp(log(mapAtt)*lambda);
    mapAttPowerNorm= mapAttPower./sum(mapAttPower(:));
    mapInhibPower  = exp(log(mapInhib)*gamma);%to the power gamma, slightly faster than .^
    mapInhibPowerNorm  = mapInhibPower./sum(mapInhibPower(:));
    u              = mapAttPowerNorm-inhibStrength*mapInhibPowerNorm; % other sign than in paper u>0 captchures attention u<0 impossible
    ustar          = u;
    %% choose target
    ustar = max(ustar,0);
    if nargout >2
        ustarSum = sum(ustar(:));
    end
    if sum(ustar(:)) > 0
        ustar              = ustar./sum(ustar(:)); % to get sum 1 for easy drawing of a fixation
    else % if zero activity use uniform prediction (this actually happens for one subject once...
        ustar = repmat(1./im_size(1)./im_size(2),im_size(1),im_size(2));
    end
    uFinal              = (1-zeta)*ustar + zeta/prod(im_size);
    
    if size(fixations,1) < ichoice+1 || isnan(fixations(ichoice+1,1)) % fixation not given
        % draw random fixation
        r              = rand;
        cu             = cumsum(uFinal(:));
        id             = find(cu>=r,1,'first');
        [fixy,fixx]   = ind2sub(im_size,id);
        fixations(ichoice+1,:)= [positionx(fixx),positiony(fixy)];
    else % fixation given
        %fixation      = fixations(ichoice+1,1:2); % is taken later
        [~,fixy]      = min((positiony-fixations(ichoice+1,2)).^2);
        [~,fixx]      = min((positionx-fixations(ichoice+1,1)).^2);
    end
    if ~isempty(fixx) && ~isempty(fixy)
        likelihood(ichoice)  = uFinal(fixy,fixx);
        fixidx               = [fixidx;fixx,fixy];
    else
        likelihood(ichoice) = 1; % for fixations outside the window... which are ignored
        if isempty(fixx)
            fixx = NaN;
        end
        if isempty(fixy)
            fixy = NaN;
        end
        fixidx            = [fixidx;fixx,fixy];
    end
    if nargout>2
        LGradFix = zeros(8,1);
        dustar = ones(size(u));
        dustar(u<0) = 0; % is almost 0/1 for the values of eta we fitted
        
        AdSigmaFix = -rad./sigmaAttention.^3.*gaussAttention.*saliency;
        FdSigmaFix = -rad./sigmaInhib.^3.*gaussInhib;
        AdSigma = (1-exp(-time.*rho)).*(AdSigmaFix./salNormalizer-salFixation.*sum(AdSigmaFix(:))./salNormalizer)+exp(-time.*rho).*AdSigma;
        FdSigma = (1-exp(-time.*omega)).*(FdSigmaFix-gaussInhib.*sum(FdSigmaFix(:)))+exp(-time.*omega).*FdSigma;
        mapGammaDerivative = -inhibStrength.*(mapInhibPowerNorm .*(log(mapInhib)-sum(log(mapInhib(:)).*mapInhibPowerNorm(:))));
        mapLambdaDerivative = mapAttPowerNorm .*(log(mapAtt)-sum(log(mapAtt(:)).*mapAttPowerNorm(:)));
        mapSigmaDerivative = -mapAttPowerNorm./sum(mapAttPower(:)).*lambda.*sum(mapAtt(:).^(lambda-1).*AdSigma(:))+lambda*mapAtt.^(lambda-1)./sum(mapAttPower(:)).*AdSigma;
        mapSigmaIDerivative = inhibStrength.*gamma.*(mapInhibPowerNorm./(sum(mapInhibPower(:))./sum(mapInhib(:).^(gamma-1).*FdSigma(:)).*maxInhib)-mapInhib.^(gamma-1)./sum(mapInhibPower(:)).*FdSigma./maxInhib);
        mapOmegaDerivative = -mapAttPowerNorm./sum(mapAttPower(:)).*lambda.*sum(mapAtt(:).^(lambda-1).*AdOmega(:))+lambda*mapAtt.^(lambda-1)./sum(mapAttPower(:)).*AdOmega;
        mapOmegaIDerivative = inhibStrength.*gamma*(mapInhibPowerNorm./(sum(mapInhibPower(:))./sum(mapInhib(:).^(gamma-1).*FdOmega(:)).*maxInhib)-mapInhib.^(gamma-1)./sum(mapInhibPower(:)).*FdOmega./maxInhib);
        
        LGradFix(1) = mapOmegaIDerivative(fixy,fixx).*dustar(fixy,fixx)./ustarSum - ustar(fixy,fixx).*sum(dustar(:).*mapOmegaIDerivative(:))./ustarSum;
        LGradFix(2) = mapOmegaDerivative(fixy,fixx) .*dustar(fixy,fixx)./ustarSum - ustar(fixy,fixx).*sum(dustar(:).*mapOmegaDerivative(:)) ./ustarSum;
        LGradFix(3) = mapSigmaDerivative(fixy,fixx) .*dustar(fixy,fixx)./ustarSum - ustar(fixy,fixx).*sum(dustar(:).*mapSigmaDerivative(:)) ./ustarSum;
        LGradFix(4) = mapSigmaIDerivative(fixy,fixx).*dustar(fixy,fixx)./ustarSum - ustar(fixy,fixx).*sum(dustar(:).*mapSigmaIDerivative(:))./ustarSum;
        LGradFix(5) = mapGammaDerivative(fixy,fixx) .*dustar(fixy,fixx)./ustarSum - ustar(fixy,fixx).*sum(dustar(:).*mapGammaDerivative(:)) ./ustarSum;
        LGradFix(6) = mapLambdaDerivative(fixy,fixx).*dustar(fixy,fixx)./ustarSum - ustar(fixy,fixx).*sum(dustar(:).*mapLambdaDerivative(:))./ustarSum;
        LGradFix(7) = -((dustar(fixy,fixx).*mapInhibPowerNorm(fixy,fixx))-ustar(fixy,fixx).*sum(dustar(:).*mapInhibPowerNorm(:)))./ustarSum;
        LGradFix(8) = (1/prod(im_size)-ustar(fixy,fixx));
        if any(isnan(LGradFix))
            LGradFix = NaN(9,1);
            L = -inf;
        end
        LGradFix(1:7) = (1-zeta).* LGradFix(1:7);
        LGrad = LGrad+LGradFix./uFinal(fixy,fixx);
    end
    
    if nargout>3
        mapsA(:,:,ichoice) = mapAtt;
        mapsI(:,:,ichoice) = mapInhib.*maxInhib;
        us(:,:,ichoice)    = uFinal;
    end
    ichoice = ichoice+1;
    fixation = fixations(ifix,:);
end
if isfinite(trialTime)
    times = times(:);
    fixations = [fixations,times];
else
    fixations = fixations(~isnan(fixations(:,1)),:);
    fixations = [fixations,times(:)];
end