
library(mgcv)
library(jpeg)
library(spatstat)
library(ggplot2)
library(grid)
library(reshape2)


#setwd('~/Documents/dynamic-saliency-from-early-vision/results/ModelSimulationsDivisive')
#setwd('~/Documents/dynamic-saliency-from-early-vision/results/ModelSimulationsTEMP')


pdfout = F

f = 5    # sample image (from 1..15)
trsel = 7   # sample trial (from 1..35 subjects)
imrange <- read.table("imrange.dat",header=TRUE)
xrange <- imrange$xrange
yrange <- imrange$yrange
sigma <- 1.05
minlen <- 10
maxlen <- 100
rmin <- 0.5
rmax <- 5
sigmaPCF = 3.8

if ( pdfout ) pdf("FigPCF.pdf",paper='a4r',width=10,height=8)

grid.newpage()
pushViewport(viewport(layout=grid.layout(2,2)))



#------------------------------------
# 1. Pair correlation functions 
# (a) original data
#------------------------------------
# PCF for experimental data
#File<-sprintf("Fixpos/Fixpos_%d.dat",f)

for (iModel in 1:5){
  
  for (f in 1:30){
    switch(iModel,
           {
             File<-sprintf("fixloc_1/fixloc_%d.dat",f)
             Im<-read.table(File,sep='\t',head=TRUE)},
           {
             File<-sprintf("ModelSimulationsTEMP/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           },
           {
             File<-sprintf("ModelSimulationsDivisive/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           },
           {
             File<-sprintf("ModelSimulationsNoInhib/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           },
           {
             File<-sprintf("ModelSimulationsSaliency/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           }
    )
    Im$image <- f
    if (f==1) {
      ImAll<-Im
    } else {
      ImAll <- rbind(ImAll,Im)
    }
  }
  
  alvsAll <- c()
  alvtAll <- c()
  alvrAll <- c()
  anum = 0
  for (f in 1:30) {
    anum = anum + 1
    Im <- subset(ImAll,image==f)
    xpos <- Im$xpos
    ypos <- Im$ypos
    fdur <- Im$fdur
    numA = length(xpos)
    A <- ppp(xpos, ypos, xrange, yrange)
    lambda <- density(A,sigmaPCF)
    
    idx = which(diff(Im$fixation)<0)
    i1 = c(1,idx+1)
    i2 = c(idx,dim(Im)[1])
    Ntrials = length(i1)
    alvr = c()
    alvt = c()
    edev = c()
    for ( t in 1:Ntrials ) {
      d <- Im[i1[t]:i2[t],]
      if ( length(d$xpos)>maxlen ) {
        d <- d[(1:maxlen),]
      }
      if ( length(d$xpos)>=minlen ) {
        xpos <- d$xpos
        ypos <- d$ypos
        fdur <- d$fdur
        numX = length(xpos)
        X <- ppp(xpos, ypos, xrange, yrange)
        g <- pcfinhom(X,lambda,kernel = "epanechnikov") 
        g$trans = g$trans*numA/numX
        indx <- which(rmin<=g$r & g$r<=rmax)
        ginteg <- sum((g$trans[indx]-1)^2)/(rmax-rmin)
        edev <- c(edev,ginteg)
        alvt = rbind(alvt,g$trans)
        alvr = rbind(alvr,g$r)
      }
    }
    alvtAll <- rbind(alvtAll,colMeans(alvt))
    alvsAll <- rbind(alvsAll,rep(anum,length(g$trans)))
    alvrAll <- rbind(alvrAll,colMeans(alvr))
  }
  pcfsOneModel <- data.frame(alvtAll)
  pcfsOneModel$image <- 1:30
  switch(iModel,
    {
      pcfsOneModel$model <- "data"
    },
    {
      pcfsOneModel$model <- "SceneWalk"
    },
    {
      pcfsOneModel$model <- "Divisive"
    },
    {
      pcfsOneModel$model <- "NoInhib"
    },
    {
      pcfsOneModel$model <- "Saliency"
    })
  if(iModel>1){
    pcfs<-rbind(pcfs,pcfsOneModel)
  } else {
    pcfs<- pcfsOneModel
  }
}

r <- g$r
pcfmelt <- melt(pcfs, id.vars = c("image","model"))
pcfmelt$r <- rep(g$r, each=nrow(pcfmelt)/nrow(g))

pcfmelt$model <- factor(pcfmelt$model,levels=c("data","SceneWalk","Divisive","NoInhib","Saliency"))

  
g <- ggplot(pcfmelt,aes(x=r,y=value))+ 
  geom_line(data=subset(pcfmelt,model=="data"),aes(x=r,y=value,group=image),colour="grey75")+
  stat_summary(data = pcfmelt,fun.y="mean", geom="line",aes(linetype = model,colour = model),size=2.5)+
  xlim(c(0,6)) +
  coord_cartesian(ylim = c(0, 3.5))+
  xlab("Distance  r [deg]") + ylab("Pair correlation  g(r)")+
  theme(text = element_text(size=20)) +
  theme(axis.title = element_text(size=25)) +
  theme(axis.text = element_text(size=20))+
  theme(axis.title.y = element_text(margin=margin(0,20,0,0))) +
  theme(axis.title.x = element_text(margin=margin(20,0,0,0))) +
  scale_linetype_manual(values = c("solid","52","42","32","13")) +
  scale_colour_manual(values = c("#FF866D","#838500","#009F4D","#0080F6","#D73BD3")) +
  guides(colour = guide_legend(override.aes = list(linetype = c("solid","52","42","32","12")),title=NULL,keywidth=5),linetype = FALSE) +
  theme(legend.position=c(.85, .85)) 
print(g)


#------------------------------------
# 2. Saccade Lenghts
#------------------------------------

for (iModel in 1:5){
  for (f in 1:30){
    switch(iModel,
           {
             File<-sprintf("fixloc_1/fixloc_%d.dat",f)
             Im<-read.table(File,sep='\t',head=TRUE)
           },
           {
             File<-sprintf("ModelSimulationsTEMP/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           },
           {
             File<-sprintf("ModelSimulationsDivisive/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           },
           {
             File<-sprintf("ModelSimulationsNoInhib/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           },
           {
             File<-sprintf("ModelSimulationsSaliency/fixloc_%d.dat",f)
             ModelData <- read.table(File,header=FALSE)
             colnames(ModelData) <- c("xpos","ypos","fdur","subj","trial","fixation")
             Im<-ModelData
           }
    )
    Im$image <- f
    switch(iModel,
           {
             Im$model <- "data"
           },
           {
             Im$model <- "SceneWalk"
           },
           {
             Im$model <- "Divisive"
           },
           {
             Im$model <- "NoInhib"
           },
           {
             Im$model <- "Saliency"
           })
    if (f==1 & iModel ==1) {
      ImAll<-Im
    } else {
      ImAll <- rbind(ImAll,Im)
    }
  }
}
Im<-ImAll

xdif <- diff(Im$xpos)
ydif <- diff(Im$ypos)
dif <- sqrt(xdif^2+ydif^2)

Im$dif = c(0,dif)
Im <- subset(Im,fixation>1)

Im$model <- factor(Im$model,levels=c("data","SceneWalk","Divisive","NoInhib","Saliency"))

g <- ggplot(data = Im,aes(x=dif)) + 
  geom_histogram(data = subset(Im,model=="data"),binwidth=.5, aes(y = ..density..),colour="grey50",fill="grey80") + 
  geom_line(aes(colour= model,linetype = model, y = ..density..),size=2,stat="density") +
  xlab("Saccade Length [deg]") + ylab("Frequency/Density") +
  xlim(c(0,30))+ylim(c(0,0.14))+
  theme(text = element_text(size=20)) +
  theme(axis.title = element_text(size=25)) +
  theme(axis.text = element_text(size=20))+
  theme(axis.title.y = element_text(margin=margin(0,20,0,0))) +
  theme(axis.title.x = element_text(margin=margin(20,0,0,0))) +
  guides(colour=guide_legend(title=NULL)) +
  scale_linetype_manual(values = c("solid","52","42","32","13")) +
  scale_colour_manual(values = c("#FF866D","#838500","#009F4D","#0080F6","#D73BD3")) +
  guides(colour = guide_legend(override.aes = list(linetype = c("solid","52","42","32","12")),title=NULL,keywidth=5),linetype = FALSE) +
  theme(legend.position=c(.85, .85))
print(g)

