function [x,L] = optim(thread,theta0,imageIDs,subIDs)


if ~exist('imageIDs','var') || isempty(imageIDs)
    imageIDs = [1:5,16:20];
end

if ~exist('subIDs','var') || isempty(subIDs)
    subIDs = 1:15;
end

fhandle = @(theta) likelihood_optim(theta,imageIDs,subIDs);
op = optimset('fminsearch');
op.Display = 'iter';
%op.OutputFcn = @(x, optimValues, state) outFunction(x, optimValues, state);
op.MaxFunEvals = 25000;
op.MaxIter = 2000;
op.TolX = 0.0001;
op.TolFun = 0.0001;

[x,L] = fminsearch(fhandle,theta0,op);




function L = likelihood_optim(theta,imageIDs,subIDs)        
    if isfinite(prior(theta,2))
        L = -sum(sum(dynamic_likelihood(theta,2,imageIDs,subIDs)))+prior(theta,2);
    else
        L = inf;
    end
end
function stop = outFunction(x, optimValues, state)
    stop = false;
    if length(x)== 8 
        x = [x(1:4);0;x(5:end)];
    elseif length(x)== 7 
        x = [x(1:4);0;0;x(5:end)];
    end
    if ~mod(optimValues.iteration,100)
        clf;
    end
    
    filename = sprintf('pOptim%d',thread);
    Lx = optimValues.fval;
    save(filename,'x','Lx');
    m = ceil(sqrt(length(x)+1));
    n = ceil(length(x)/m);
    for iPar = 1:length(x)
        subplot(n,m,iPar);
        hold on
        plot(optimValues.iteration,exp(x(iPar)),'k.')
        xlabel('iteration');
        switch iPar
            case 1
                ylabel('\omega');
            case 2
                ylabel('\rho');
            case 3
                ylabel('\sigma_A')
            case 4
                ylabel('\sigma_I');
            case 5
                ylabel('\gamma');
            case 6
                ylabel('\lambda')
            case 7
                ylabel('\eta')
            case 8
                ylabel('C_{inhib}');
            case 9
                ylabel('\zeta')
        end
    end
    subplot(n,m,n*m)
    hold on
    plot(optimValues.iteration,optimValues.fval,'k.')
    xlabel('iteration');
    ylabel('-log-Likelihood');
    drawnow;
end

end
