function [x,L] = optim_noInhib(thread,theta0,imageIDs,subIDs)


if ~exist('imageIDs','var') || isempty(imageIDs)
    imageIDs = [1:5,16:20];
end

if ~exist('subIDs','var') || isempty(subIDs)
    subIDs = 1:15;
end

fhandle = @(theta) likelihood_optim(theta,imageIDs,subIDs);
op = optimset('fminsearch');
op.Display = 'iter';
op.OutputFcn = @(x, optimValues, state) outFunction(x, optimValues, state);
op.MaxFunEvals = 5000;


[x,L] = fminsearch(fhandle,theta0,op);




function L = likelihood_optim(theta,imageIDs,subIDs)
    L = -sum(sum(dynamic_likelihood_noInhib(theta,2,imageIDs,subIDs)))+prior(theta);
end
function stop = outFunction(x, optimValues, state)
    stop = false;
    filename = sprintf('pOptimNoInhib%d',thread);
    Lx = optimValues.fval;
    save(filename,'x','Lx');
    m = ceil(sqrt(length(x)+1));
    n = ceil(length(x)/m);
    for iPar = 1:length(x)
        subplot(n,m,iPar);
        hold on
        plot(optimValues.iteration,exp(x(iPar)),'k.')
        xlabel('iteration');
        switch iPar
            case 1
                ylabel('\rho');
            case 2
                ylabel('\sigma_A')
            case 3
                ylabel('\lambda')
            case 4
                ylabel('\zeta')
        end
           
    end
    subplot(n,m,n*m)
    hold on
    plot(optimValues.iteration,optimValues.fval,'k.')
    xlabel('iteration');
    ylabel('-log-Likelihood')
    drawnow
end

end