function plot_Hierarchical(samples,hypersamples)
%function plot_parsV(parsV)

pars = squeeze(mean(samples));
hyperpars = mean(hypersamples);

if ~exist('subIDs','var') || isempty(subIDs)
    subIDs = 1:size(pars,1);
end

%% calculate fixations from model and mean lengths per subject

options.targetSelector = 3;
options.normalizeInhib = true;
options.nonzero = true;

pars0 = getMLPars;
pars0 = exp(pars0);

dataset = 1;
    
SLAll = [];
SLAllM = [];
for im = 1:30
    [fix,saliency,deg_size] = read_data(dataset,im);
    kSub = 0;
    for sub = subIDs
        kSub = kSub+1;
        pSub = pars0;
        pSub([3,4]) = pars(kSub,:);
        pSub(3) = pSub(3) .* sqrt(pSub(6));
        pSub(4) = pSub(4) .* sqrt(pSub(5));
        fixSub = fix(fix(:,4)==sub,:);
        times     = fixSub(:,3);% NOT cumsum(fixSub(:,3));
        fixations = fixSub(:,[1,2]);
        fixationsModel = dynamic_model_derivative([128,128],deg_size,saliency,fixations(1,:),times/1000,[],pSub,[],[],options);
        SLength = diff(fixations);
        SLength = sqrt(sum(SLength.^2,2));
        SLengthModel = diff(fixationsModel(:,1:2));
        SLengthModel = sqrt(sum(SLengthModel.^2,2));
        SLAll = [SLAll;[SLength,sub.*ones(size(SLength))]];
        SLAllM = [SLAllM;[SLengthModel,sub.*ones(size(SLengthModel))]];
    end
end

SLSub = zeros(size(subIDs));
SLSubM = zeros(size(subIDs));
kSub = 0;
for sub = subIDs
    kSub = kSub+1;
    SLSub(kSub) = mean(SLAll(SLAll(:,2)==sub,1));
    SLSubM(kSub) = mean(SLAllM(SLAllM(:,2)==sub,1));
end


%% plotting
figure
subplot(1,3,1)
hold on
for isub = 1:size(samples,2)
    scatter(samples(1000:end,isub,1),samples(1000:end,isub,2),15,'filled','MarkerFacealpha',.01);
end
plot(pars(:,1),pars(:,2),'k.','MarkerSize',15)
% Calculate the eigenvectors and eigenvalues
covariance = [hyperpars(3),hyperpars(5).*sqrt(hyperpars(3).*hyperpars(4));...
    hyperpars(5).*sqrt(hyperpars(3).*hyperpars(4)),hyperpars(4)];
[eigenvec, eigenval ] = eig(covariance);
largest_eigenvec = eigenvec(:, 1);
largest_eigenval = eigenval(1,1);
smallest_eigenvec = eigenvec(:,2);
smallest_eigenval = eigenval(2,2);

% Calculate the angle between the x-axis and the largest eigenvector
angle = atan2(largest_eigenvec(2), largest_eigenvec(1));

chisquare_val = 2.4477;
theta_grid = linspace(0,2*pi,1000);
X0=hyperpars(1);
Y0=hyperpars(2);
a=chisquare_val*sqrt(largest_eigenval);
b=chisquare_val*sqrt(smallest_eigenval);
ellipse_x_r  = a*cos( theta_grid );
ellipse_y_r  = b*sin( theta_grid );
R = [ cos(angle) sin(angle); -sin(angle) cos(angle) ];
r_ellipse = [ellipse_x_r;ellipse_y_r]' * R;

% Draw the error ellipse
plot(r_ellipse(:,1) + X0,r_ellipse(:,2) + Y0,'-k','LineWidth',2)
plot([0,14],[0,14],'k--') %equal sizes line
axis equal
axis square
box off
xlim([1,12])
ylim([1,12])
set(gca,'LineWidth',2,'TickDir','out','FontSize',16)
ylabel('\sigma_I','FontSize',20)
xlabel('\sigma_A','FontSize',20)

subplot(1,3,2)
% Predicted saccade length vs fitted attention span
plot(pars(:,1),SLSubM,'k.','MarkerSize',15)
hold on
P = polyfit(pars(:,1),SLSubM',1);
plot([3,9.5],P(1)*[3,9.5]+P(2),'k','LineWidth',2)
text(2.5,8.5,sprintf('y = %.3fx + %.3f',P(1),P(2)),'FontSize',14)
axis square
box off
set(gca,'LineWidth',2,'TickDir','out','FontSize',16)
ylabel('Predicted Saccade Length [°]','FontSize',18)
xlabel('Mean \sigma_A [°]','FontSize',20)


subplot(1,3,3)
% Predicted vs measured saccade length
plot(SLSub,SLSubM,'k.','MarkerSize',15)
hold on
plot([4,10],[4,10],'k--','LineWidth',2);

P = polyfit(SLSub,SLSubM,1);
plot([4,10],P(1)*[4,10]+P(2),'k','LineWidth',2)

%xlim([4,8])
%ylim([4,8])
axis square
box off
set(gca,'LineWidth',2,'TickDir','out','FontSize',16)
xlabel('Measured Saccade Length [°]','FontSize',18)
ylabel('Predicted Saccade Length [°]','FontSize',18)
text(4.5,8.5,sprintf('y = %.3fx + %.3f',P(1),P(2)),'FontSize',14)
set(gcf,'Position',[150,650,1500,450])
end