function [fix,saliency,deg_size] = read_data(dataset,im)
% function [fix,saliency,par] = read_data(dataset,im)
% reads fixations, saliency map and parameters for a specific experiment to
% run the model on. Im is the number of the image to run the model on.
% Possible inputs for dataset are:
%
% 1: SpatStat1_1, SpatStat1    Hans first dataset on natural images
% 2: SpatStat1_2               Hans first dataset on natural images second viewing
% 3: SpatStat2_1, SpatStat2    Lars dataset with extreme starting position
% 11-15: SpatStat1             Saliency maps & data for crossvalidation,
%                              i.e. based on less data.
% 101: ModelSpatStat1_1        Model simulated data for 1
%
% Missing are: SpatStat3, Cebi1, Cebi2, Corpus data
% 
% We employ persistent variables for SpatStat 1 to improve speed
% performance


persistent fixSave
persistent salSave
if isempty(fixSave)
    fixSave = cell(30,1);
end
if isempty(salSave)
    salSave = cell(30,1);
end

switch dataset
    case {1,'SpatStat1_1','SpatStat1'}
        if isempty(fixSave{im})
            fixSave{im} = importdata(sprintf('fixloc_1/fixloc_%d.dat',im),'\t');
        end
        if isempty(salSave{im,1})
            salSave{im,1}  = load(sprintf('modelInputCross01/fx_%d.dat',im)); % based on only the first 15 subjects
        end
        %par       = importdata(sprintf('../../../../results/modelInputCross/fx_%d.par',im));
        fix = fixSave{im};
        saliency = salSave{im};
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
    case {2,'SpatStat1_2'}
        fix       = importdata(sprintf('fixloc_2/fixloc_%d.dat',im),'\t');
        saliency  = load(sprintf('modelInputCross01/fx_%d.dat',im)); % based on only the first 15 subjects
        par       = importdata(sprintf('modelInputCross01/fx_%d.par',im));
    case {11}
        if isempty(fixSave{im})
            fixSave{im} = importdata(sprintf('fixloc_1/fixloc_%d.dat',im),'\t');
        end
        if (size(salSave,2)<2) || isempty(salSave{im,2}) 
            salSave{im,2}  = load(sprintf('modelInputCross01/fx_%d_1.dat',im)); 
        end
        fix = fixSave{im};
        saliency = salSave{im,2};
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
    case {12}
        if isempty(fixSave{im})
            fixSave{im} = importdata(sprintf('fixloc_1/fixloc_%d.dat',im),'\t');
        end
        if (size(salSave,2)<3) || isempty(salSave{im,3})
            salSave{im,3}  = load(sprintf('modelInputCross01/fx_%d_2.dat',im)); 
        end
        fix = fixSave{im};
        saliency = salSave{im,3};
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
    case {13}
        if isempty(fixSave{im})
            fixSave{im} = importdata(sprintf('fixloc_1/fixloc_%d.dat',im),'\t');
        end
        if (size(salSave,2)<4) || isempty(salSave{im,4})
            salSave{im,4}  = load(sprintf('modelInputCross01/fx_%d_3.dat',im)); 
        end
        fix = fixSave{im};
        saliency = salSave{im,4};
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
    case {14}
        if isempty(fixSave{im})
            fixSave{im} = importdata(sprintf('fixloc_1/fixloc_%d.dat',im),'\t');
        end
        if (size(salSave,2)<5) || isempty(salSave{im,5})
            salSave{im,5}  = load(sprintf('modelInputCross01/fx_%d_4.dat',im)); 
        end
        fix = fixSave{im};
        saliency = salSave{im,5};
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
    case {15}
        if isempty(fixSave{im})
            fixSave{im} = importdata(sprintf('fixloc_1/fixloc_%d.dat',im),'\t');
        end
        if (size(salSave,2)<6) || isempty(salSave{im,6})
            salSave{im,6}  = load(sprintf('modelInputCross01/fx_%d_5.dat',im));
        end
        fix = fixSave{im};
        saliency = salSave{im,6};
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
    case {101,'ModelSpatStat1_1'}
        fix       = importdata(sprintf([useModelData,'fixloc_%d.dat'],im),'\t');
        saliency  = load(sprintf('modelInputCross01/fx_%d.dat',im)); % based on only the first 15 subjects
        par = struct;
        par.data =   [128;128;1.0359;32.1116;0.8287;25.6893];
end
if isstruct(fix)
    fix       = fix.data;
end

deg_size  = [par.data(6)-par.data(5),par.data(4)-par.data(3)];
fix(:,1)  = fix(:,1)-par.data(3);% image coordinates
fix(:,2)  = fix(:,2)-par.data(5);