function runModelAndSave_noinhib(imageIDs,pars,type,directory,inputDirectory,inputSaliency)
%function runModelAndSave_divisive(imageIDs,pars,type,directory,inputDirectory,inputSaliency)
% this function runs the model with given parameters and saves the data in
% a R readable format in the given directory.
% using type you may choose how the number of fixations or timings are
% chosen:
% 1 = from the gamma distribution, 10 seconds long
% 2 = from the gamma distribution, same number of fixations
% 3 = times as for the subjects
%
% With inputDirectory and inputSaliency you may choose the place to look
% for the fixation location data and the saliency maps respectively

oldDir = cd(fileparts(which('runModelAndSave_noinhib.m')));


if ~exist('directory','var') || isempty(directory)
    directory = 'ModelSimulationsNoInhib/';
end

if ~exist('type','var') || isempty(type)
    type = 3;
end

modelType = 2;

if ~exist('inputDirectory','var') || isempty(inputDirectory)
    inputDirectory = 'fixloc_1';
end

if ~exist('inputSaliency','var') || isempty(inputSaliency)
    inputSaliency = 'modelInputCross';
end

%% options

options = struct;
switch modelType
    case 1  % Model Engbert et al 2015
        options.targetSelector = 1;
        options.normalizeInhib = true;
        options.nonzero = false;
    case 2  % full model 
        options.targetSelector = 3;
        options.normalizeInhib = true;
        options.nonzero = true;
end
%% run
if ~exist(directory,'dir')
    mkdir(directory);
end
for imageID = imageIDs
    fix       = importdata(sprintf(fullfile(inputDirectory,'fixloc_%d.dat'),imageID),'\t');
    fix       = fix.data;
    saliency  = load(sprintf(fullfile(inputSaliency,'fx_%d.dat'),imageID));
    deg_size  = [25.68927-0.8286862,32.11159-1.035858];% collected from file par.
    if type ==2 || type == 3 || type == 5 || type == 6
        fixations = nan(size(fix));
    else
        fixations = [];
    end
    for sub = unique(fix(:,4))'
        fixSub = fix(fix(:,4)==sub,:);
        times     = fixSub(:,3);% NOT cumsum(fixSub(:,3));
        switch type
            case 1
                [f,l] = dynamic_model_noInhib([128,128],deg_size,saliency,deg_size/2,[],[],pars,10,[]);
                fixations = [fixations;[f(2:end,:),repmat(sub,size(f,1)-1,1),repmat(fixSub(1,5),size(f,1)-1,1),(2:size(f,1))']];
            case 2
                [f,l] = dynamic_model_noInhib([128,128],deg_size,saliency,deg_size/2,[],[],pars,[],size(fixSub,1));
                fixations(fix(:,4)==sub,1:3) = f(1:end,1:3);
            case 3
                [f,l] = dynamic_model_noInhib([128,128],deg_size,saliency,deg_size/2,times/1000,[],pars,[],[]);
                fixations(fix(:,4)==sub,1:3) = f(1:end,1:3);
            case 4
                [f,l] = dynamic_model_noInhib([128,128],deg_size,saliency,fixSub(1,1:2),[],[],pars,10,[]);
                fixations = [fixations;[f(2:end,:),repmat(sub,size(f,1)-1,1),repmat(fixSub(1,5),size(f,1)-1,1),(1:(size(f,1)-1))']];
            case 5
                times = [1000+times(1);nan(size(times,1)-1,1)];
                [f,l] = dynamic_model_noInhib([128,128],deg_size,saliency,fixSub(1,1:2),times/1000,[],pars,[],size(fixSub,1),options);
                fixations(fix(:,4)==sub,1:3) = f(2:end,1:3);
            case 6
                times(1) = times(1)+1000;
                [f,l] = dynamic_model_noInhib([128,128],deg_size,saliency,fixSub(1,1:2),times/1000,[],pars,[],[]);
                fixations(fix(:,4)==sub,1:3) = f(:,1:3);
        end
        if type ==2 || type == 3 || type == 5 || type == 6
            fixations(fix(:,4)==sub,4:6) = fixSub(:,4:6);
        end
        
    end
    fixations(:,3) = fixations(:,3)*1000;
    fixations(:,1) = fixations(:,1)+1.035858; %Add border of the image
    fixations(:,2) = fixations(:,2)+0.8286862; 
    %csvwrite_with_headers(fullfile(directory,sprintf('fixloc_%d.dat',imageID)),fixations,{'"xpos"','"ypos"','"fdur"','"subj"','"trial"','"fixation"'})
    dlmwrite(fullfile(directory,sprintf('fixloc_%d.dat',imageID)),fixations,'\t');
    fprintf('finished image Nr %d\n',imageID);
end

cd(oldDir);