function runModelAndSave_saliency(imageIDs,directory,inputDirectory,inputSaliency)
%function runModelAndSave(imageIDs,pars,type,directory,inputDirectory,inputSaliency,modelType)
% this function runs the model with given parameters and saves the data in
% a R readable format in the given directory.
% using type you may choose how the number of fixations or timings are
% chosen:
% 1 = from the gamma distribution, 10 seconds long
% 2 = from the gamma distribution, same number of fixations
% 3 = times as for the subjects
%
% With inputDirectory and inputSaliency you may choose the place to look
% for the fixation location data and the saliency maps respectively
%
% In modelType you can choose from the different versions of the model.
% 1 = original model
% 2 = full model with nonzero initialization and better eta handling

oldDir = cd(fileparts(which('runModelAndSave.m')));


if ~exist('directory','var') || isempty(directory)
    directory = 'ModelSimulationsSaliency';
end

if ~exist('inputDirectory','var') || isempty(inputDirectory)
    inputDirectory = 'fixloc_1';
end

if ~exist('inputSaliency','var') || isempty(inputSaliency)
    inputSaliency = 'modelInputCross';
end

%% options

%% run
if ~exist(directory,'dir')
    mkdir(directory);
end
for imageID = imageIDs
    fix       = importdata(sprintf(fullfile(inputDirectory,'fixloc_%d.dat'),imageID),'\t');
    fix       = fix.data;
    saliency  = load(sprintf(fullfile(inputSaliency,'fx_%d.dat'),imageID));
    deg_size  = [25.68927-0.8286862,32.11159-1.035858];% collected from file par.
    im_size   = [128,128];
    positionx = linspace(0,1,im_size(2)).*deg_size(2);
    positiony = (linspace(0,1,im_size(1)).*deg_size(1))';
    fixations = fix;
    for sub = unique(fix(:,4))'
        fixSub = fix(fix(:,4)==sub,:);
        f = fixSub(:,1:2);
        for iChoice = 2:size(fixSub,1)
            r              = rand;
            cu             = cumsum(saliency(:));
            id             = find(cu>=r,1,'first');
            [fixy,fixx]   = ind2sub(im_size,id);
            f(iChoice,:)= [positionx(fixx),positiony(fixy)];
        end
        fixations(fix(:,4)==sub,1:2) = f(:,1:2);
    end
    fixations(:,1) = fixations(:,1)+1.035858; %Add border of the image
    fixations(:,2) = fixations(:,2)+0.8286862;
    %csvwrite_with_headers(fullfile(directory,sprintf('fixloc_%d.dat',imageID)),fixations,{'"xpos"','"ypos"','"fdur"','"subj"','"trial"','"fixation"'})
    dlmwrite(fullfile(directory,sprintf('fixloc_%d.dat',imageID)),fixations,'\t');
    fprintf('finished image Nr %d\n',imageID);
end

cd(oldDir);