library(R.matlab)
library(mcmc)
library(coda)
library(mcmcplots)
library(ggplot2)


s1<-readMat('samplesGeneric001.mat')
s2<-readMat('samplesGeneric002.mat')
s3<-readMat('samplesGeneric003.mat')
s1s <- s1$samples[,1:8]
s2s <- s2$samples[,1:8]
s3s <- s3$samples[,1:8]
colnames(s1s) <- c('decayInhib','decayAttention','sigmaAtt','sigmaInhib','gamma','lambda','scaleInhibition','zeta') #,'loglikelihood')
colnames(s2s) <- c('decayInhib','decayAttention','sigmaAtt','sigmaInhib','gamma','lambda','scaleInhibition','zeta') #,'loglikelihood')
colnames(s3s) <- c('decayInhib','decayAttention','sigmaAtt','sigmaInhib','gamma','lambda','scaleInhibition','zeta') #,'loglikelihood')

s1s <- exp(s1s)
s2s <- exp(s2s)
s3s <- exp(s3s)
s1mcmc <- mcmc(s1s[1000+5*(1:(49000/5)),],thin=5, start=1000)
s2mcmc <- mcmc(s2s[1000+5*(1:(49000/5)),],thin=5, start=1000)
s3mcmc <- mcmc(s3s[1000+5*(1:(49000/5)),],thin=5, start=1000)
s <- mcmc.list(s1mcmc,s2mcmc,s3mcmc)

summary(s)
effectiveSize(s)
#plot(s)
#parcorplot(s)
parcorplot(s,c('decayInhib','decayAttention','sigmaAtt','sigmaInhib','gamma','lambda','scaleInhibition','zeta'))
#mcmcplot(s)


##  ggplot plots


sDataFrame <- as.data.frame(as.matrix(s))


t <- theme(axis.title = element_text(size=30)) +
  theme(axis.title.y = element_text(vjust=1)) +
  theme(axis.text = element_text(size=24))  +
  theme(panel.grid = element_line(color = "white"))


g1 <- ggplot(data = sDataFrame)+ aes(x=decayInhib)+ t + 
  geom_histogram(binwidth=.01,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Decay inhibition [1/s]") + ylab("")

g2 <- ggplot(data = log10(sDataFrame))+ aes(x=decayAttention)+ t + 
  geom_histogram(binwidth=1,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Log_10 decay attention [1/s]") + ylab("")

g3 <- ggplot(data = sDataFrame)+ aes(x=sigmaAtt)+ t + 
  geom_histogram(binwidth=.005,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Sigma attention [°]") + ylab("")

g4 <- ggplot(data = sDataFrame)+ aes(x=sigmaInhib)+ t + 
  geom_histogram(binwidth=.01,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Sigma inhibtion [°]") + ylab("")

g5 <- ggplot(data = log10(sDataFrame))+ aes(x=gamma)+ t + 
  geom_histogram(binwidth=.25,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("log_10 Gamma") + ylab("")

g6 <- ggplot(data = sDataFrame)+ aes(x=lambda)+ t + 
  geom_histogram(binwidth=.002,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Lambda") + ylab("")

g7 <- ggplot(data = sDataFrame)+ aes(x=scaleInhibition)+ t + 
  geom_histogram(binwidth=.0015,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Scalefactor inhibition C_I") + ylab("") 


g8 <- ggplot(data = sDataFrame)+ aes(x=zeta)+ t + 
  geom_histogram(binwidth=.0005,colour= "black",aes(y = ..density..)) + 
  geom_density(colour="black",size=1.5)  +
  xlab("Zeta") + ylab("") 

plot(g1)
plot(g2)
plot(g3)
plot(g4)
plot(g5)
plot(g6)
plot(g7)
plot(g8)


### Scatter Plot example

gscatter1 <- ggplot(data = sDataFrame) +aes(x = scaleInhibition, y = sigmaInhib) +
  stat_bin2d (bins=50,aes(alpha= ..count..)) +
  scale_fill_continuous(low = "black",high = "black",limits=c(1, 5000), space="Lab",guide=FALSE)+t+
  xlab("") + ylab("") + scale_y_continuous(name = "", breaks=c(4.4,4.5,4.6,4.7))
  #xlab("Scale inhibition") + ylab("Sigma inhibition [°]")

plot(gscatter1)

gscatter2 <- ggplot(data = sDataFrame) +aes(x = sigmaAtt, y = sigmaInhib) +
  stat_bin2d (bins=50,aes(alpha= ..count..)) +
  scale_fill_continuous(low = "black",high = "black",limits=c(1, 5000), space="Lab",guide=FALSE)+t+
  xlab("") + ylab("") + 
  scale_x_continuous(name = "",breaks=c(5.8,5.9,6)) + scale_y_continuous(name = "", breaks=c(4.4,4.5,4.6,4.7))
#  xlab("Sigma excitation [°]") + ylab("Sigma inhibition [°]")

plot(gscatter2)

gscatter3 <- ggplot(data = sDataFrame) +aes(x = scaleInhibition, y = sigmaAtt) +
  stat_bin2d (bins=50,aes(alpha= ..count..)) +
  scale_fill_continuous(low = "black",high = "black",limits=c(1, 1000), space="Lab",guide=FALSE)+t+
  xlab("") + ylab("") + scale_y_continuous(name = "",breaks=c(5.8,5.9,6))
#  xlab("Scale inhibition") + ylab("Sigma attention [°]")

plot(gscatter3)