function [samples] = sampling_generic(k,Thread,imageIDs,subjIDs,covMat)


rng('shuffle')


if ~exist('imageIDs','var') || isempty(imageIDs)
    imageIDs = [1:5,16:20];
end
if ~exist('subjIDs','var') || isempty(subjIDs)
    subjIDs  = 1:15;
end

if isnumeric(k)
    filename = sprintf('samplesGeneric%03d',Thread);
    load GenericFitAll.mat
    p = optimresults.x+0.01*randn(8,1);
    p = p';
    samples  = NaN(k,length(p)+numel(imageIDs)*numel(subjIDs)+1);
    samples(1,:) = [p,NaN(1,numel(imageIDs)*numel(subjIDs)+1)];
    sizeFactor = 0.014;
else
    load(k)
    filename = k;
    p = samples(sum(~isnan(samples(:,1))),1:8);
    k = size(samples,1);
end
if ~exist('covMat','var') || isempty(covMat)
    covMat = eye(length(p));
end
cholesky = chol(2.4^2./(length(p))*covMat);

theta = samples(find(~isnan(samples(:,1)),1,'last'),1:8); % get parameters from last sample
start = sum(~isnan(samples(:,1)))+1;

[lalt,N] = dynamic_likelihood_derivative(theta,2,imageIDs,subjIDs);
samples(start-1,:) = [theta,sum(sum(lalt)),lalt(:)'];
save(filename,'samples','N','imageIDs','subjIDs','covMat','sizeFactor','cholesky')

Nrejected = sum(samples(1:(start-1),1)==samples(2:start,1));
for i = start:k
    accepted = false;
    theta_prop = theta + (cholesky*randn(8,1)*sizeFactor)';
    l      = dynamic_likelihood_derivative(theta_prop,2,imageIDs,subjIDs);
    accept     = exp(sum(sum(l-lalt))-prior(theta_prop,2)+prior(theta,2));
    if accept>1
        accepted = true;
    else
        if rand<accept
            accepted = true;
        else
            Nrejected=Nrejected+1;
        end
    end
    if accepted
        theta = theta_prop;
        lalt  = l;
        %sizeFactor = sizeFactor*exp(.3*50/(i+50));
    else
        %sizeFactor = sizeFactor*exp(-.1*50/(i+50));
    end
    samples(i,:) = [theta,sum(sum(lalt)),lalt(:)'];
    fprintf('%d of %d sampled, %d rejected (AR = %2.3f %%), l(%d) = %d, p = %d, SF = %d, values:\n [%.3d,%.3d,%.3d,%.3d,%.3d,%.3d,%.3d,%.3d]\n',i,k,Nrejected,100*(1-Nrejected/i),sum(sum(N)),sum(sum(l)),accept,sizeFactor,theta(1),theta(2),theta(3),theta(4),theta(5),theta(6),theta(7),theta(8));
    if ~mod(i,25)
        save(filename,'samples','N','imageIDs','subjIDs','covMat','sizeFactor','cholesky')
    end
end

end