function [samples,hypersamples] = sampling_hierarchical_Gibbs(k,Thread,imageIDs,subjIDs)


rng('shuffle')
Nrejected = 0;

if ~exist('imageIDs','var') || isempty(imageIDs)
    imageIDs = [1:5,16:20];
end
if ~exist('subjIDs','var') || isempty(subjIDs)
    subjIDs  = 1:15;
end

if isnumeric(k)
    filename = sprintf('samplesHierarchicalGibbs%03d',Thread);
    pars = getMLPars;
    pars = exp(pars);
    pars = pars([3,4]);
    p = repmat(pars(:)',length(subjIDs),1);
    hyperpars = [pars(:)',1,1,0];
    %     end
    samples  = NaN(k,length(subjIDs),2);
    samples(1,:,:) = p;
    hypersamples = NaN(k,6);
    hypersamples(1,:) = [hyperpars,NaN];
else
    load(k)
    filename = k;
    k = size(samples,1);
end
SizeFactor = 0.08;
SizeFactor2 = .3;

pars0 = getMLPars;
pars0 = exp(pars0);
    
theta = squeeze(samples(find(~isnan(samples(:,1)),1,'last'),:,:)); % get parameters from last sample

hyperpars = squeeze(hypersamples(find(~isnan(hypersamples(:,1)),1,'last'),1:(end-1)));

start = sum(~isnan(samples(:,1)))+1;

lold = zeros(size(subjIDs));
pars = pars0;
kSub = 0;
for iSub = subjIDs
    kSub = kSub+1;
    pars(3:4) = theta(kSub,:);
    lold(kSub) = sum(dynamic_likelihood_derivative(log(pars),2,imageIDs,iSub))+hyperL(theta(kSub,:),hyperpars);
end
hypersamples(start-1,:) = [hyperpars,-sum(lold)];

for i = start:k
    
    %% updating hyperparameters
    accepted = false;
    hyper_prop = hyperpars;
    hyper_prop(1) = hyper_prop(1)+4*SizeFactor.*randn(1,1);
    hyper_prop(2) = hyper_prop(2)+2*SizeFactor.*randn(1,1);
    hyper_prop(3) = hyper_prop(3)+4*SizeFactor.*randn(1,1);
    hyper_prop(4) = hyper_prop(4)+5*SizeFactor.*randn(1,1);
    hyper_prop(5) = hyper_prop(5)+SizeFactor.*randn(1);
    a = .25;
    b = 1;
    hyperPriorOld = log(1/2)+invGammaPdf(hyperpars(3),a,b)+invGammaPdf(hyperpars(4),a,b);
    hyperPrior = log(1/2)+invGammaPdf(hyper_prop(3),a,b)+invGammaPdf(hyper_prop(4),a,b);
    hlOld = hyperL(theta,hyperpars);
    hl = hyperL(theta,hyper_prop);
    accept     = exp(hl-hlOld+hyperPrior-hyperPriorOld);
    if accept>1
        accepted = true;
    else
        if rand<accept
            accepted = true;
        else
            Nrejected=Nrejected+1;
        end
    end
    if accepted
        kSub = 0;
        for iSub = subjIDs
            kSub = kSub+1;
            lold(kSub)  = lold(kSub)+hyperL(theta(kSub,:),hyper_prop)-hyperL(theta(kSub,:),hyperpars);
        end
        hyperpars = hyper_prop;
        SizeFactor = SizeFactor*exp(.3*50/(i+50));
    else
        SizeFactor = SizeFactor*exp(-.1*50/(i+50));
    end
    
    %% update individuals
    
    kSub = 0;
    for iSub = subjIDs
        kSub = kSub+1;
        theta_prop = theta(kSub,:)+SizeFactor2.*randn(1,2);
        pars(3:4) = theta_prop;
        l = sum(dynamic_likelihood_derivative(log(pars),2,imageIDs,iSub))+hyperL(theta_prop,hyperpars);
        %[l]      = H_likelihood(theta_prop,hyper_prop,2,imageIDs,subjIDs);
        if ~isfinite(l)
            fprintf('out of bounds');
        end
        accepted = false;
        accept     = exp(l-lold(kSub));
        if accept>1
            accepted = true;
        else
            if rand<accept
                accepted = true;
            else
                Nrejected=Nrejected+1;
            end
        end
        if accepted
            theta(kSub,:) = theta_prop;
            lold(kSub)  = l;
            SizeFactor2 = SizeFactor2*exp(.3*50/(i+50));
        else
            SizeFactor2 = SizeFactor2*exp(-.1*50/(i+50));
        end
    end
    samples(i,:) = theta(:);
    hypersamples(i,:) = [hyperpars,-sum(lold)];
    fprintf('%d of %d sampled, %d rejected (AR = %2.3f %%), l = %d, p = %4.2d, SF1 = %4.2d, SF2 = %4.2d, values:\n',i,k,Nrejected,100*(1-Nrejected/i/(length(subjIDs)+1)),l,accept,SizeFactor,SizeFactor2);
    disp(hyperpars);
    disp(theta);
    plot_sampling(samples,hypersamples,theta,hyperpars)
    save(filename,'samples','hypersamples','SizeFactor','SizeFactor2','Nrejected')
end

end

function plot_sampling(samples,hypersamples,theta,hyperpars)
    clf
    subplot(2,2,1)
    plot(samples(:,:,1))
    xlabel('Sample #')
    ylabel('Sigma A')
    
    subplot(2,2,2)
    plot(samples(:,:,2))
    xlabel('Sample #')
    ylabel('Sigma I')
    
    subplot(2,2,4)
    plot(hypersamples(:,6),'k','LineWidth',2)
    xlabel('Sample #')
    ylabel('Likelihood')
        
    subplot(2,2,3)
    plot(theta(:,1),theta(:,2),'k.')
    hold on
    % Calculate the eigenvectors and eigenvalues
    covariance = [hyperpars(3),hyperpars(5).*sqrt(hyperpars(3).*hyperpars(4));...
        hyperpars(5).*sqrt(hyperpars(3).*hyperpars(4)),hyperpars(4)];
    [eigenvec, eigenval ] = eig(covariance);
    largest_eigenvec = eigenvec(:, 1);
    largest_eigenval = eigenval(1,1);
    smallest_eigenvec = eigenvec(:,2);
    smallest_eigenval = eigenval(2,2);
    
    % Calculate the angle between the x-axis and the largest eigenvector
    angle = atan2(largest_eigenvec(2), largest_eigenvec(1));
          
    chisquare_val = 2.4477;
    theta_grid = linspace(0,2*pi,1000);
    X0=hyperpars(1);
    Y0=hyperpars(2);
    a=chisquare_val*sqrt(largest_eigenval);
    b=chisquare_val*sqrt(smallest_eigenval);
    ellipse_x_r  = a*cos( theta_grid );
    ellipse_y_r  = b*sin( theta_grid );
    R = [ cos(angle) sin(angle); -sin(angle) cos(angle) ];
    r_ellipse = [ellipse_x_r;ellipse_y_r]' * R;
    
    % Draw the error ellipse
    plot(r_ellipse(:,1) + X0,r_ellipse(:,2) + Y0,'-')
    
    xlabel('Sigma A')
    ylabel('Sigma I')
    drawnow
end


function hyperL = hyperL(pars,hyperpars)
    sigmaA = hyperpars(3);
    sigmaI = hyperpars(4);
    covMat = [sigmaA,hyperpars(5).*sqrt(sigmaA.*sigmaI);...
        hyperpars(5).*sqrt(sigmaA.*sigmaI),sigmaI];
    mu = hyperpars(1:2);
    mu = mu(:);
    diff = bsxfun(@minus,pars,mu');
    hyperL = -trace(diff*(covMat\diff'))./2-size(diff,1).*(log(2*pi)+log(det(covMat))/2);
end
function logp = invGammaPdf(x,a,b)
% calculates the log-likelihood for the inverse Gammapdf we need for the
% prior on the variances.
%p = b^a./gamma(a).*x.^(-a-1).*exp(-b./x);
logp = a.*log(b)-gammaln(a)-(a+1).*log(x)-b./x;
logp(x<=0) = -inf;
end